import {HTTPKernelModule} from "../HTTPKernelModule";
import {Injectable, Inject} from "../../../di";
import {uuid_v4} from "../../../util";
import {HTTPKernel} from "../HTTPKernel";
import {Request} from "../../lifecycle/Request";
import {Logging} from "../../../service/Logging";

/**
 * HTTP kernel middleware that tries to look up the session ID from the request.
 * If none exists, generates a new one and sets the cookie.
 */
@Injectable()
export class SetSessionCookieHTTPModule extends HTTPKernelModule {
    public readonly executeWithBlockingWriteback = true

    @Inject()
    protected readonly logging!: Logging

    public static register(kernel: HTTPKernel) {
        kernel.register(this).first()
    }

    public async apply(request: Request) {
        if ( !request.cookies.has('extollo.session') ) {
            const session = `${uuid_v4()}-${uuid_v4()}`

            this.logging.verbose(`Starting session: ${session}`)
            request.cookies.set('extollo.session', session)
        }
        return request
    }
}
